package de.hft.stuttgart.citydoctor2.gui.tree;

import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.ImplicitGeometryHolder;
import de.hft.stuttgart.citydoctor2.gui.CheckStatus;
import de.hft.stuttgart.citydoctor2.gui.Renderer;

public class GeometryNode extends Renderable {

	private final Geometry geom;
	private final String text;

	public GeometryNode(Geometry geom) {
		this.geom = geom;
		StringBuilder sb = new StringBuilder();
		if (geom instanceof ImplicitGeometryHolder) {
			sb.append("Implicit");
		}
		sb.append("Geometry [");
		sb.append(geom.getType());
		sb.append(", ");
		sb.append(geom.getLod());
		if (geom.getGmlId().isGenerated()) {
			sb.append("]");
		} else {
			sb.append("] ");
			sb.append(geom.getGmlId());
		}
		text = sb.toString();
	}

	@Override
	public void refreshTextColor() {
		if (!geom.isValidated()) {
			setStatus(CheckStatus.NOT_CHECKED);
		} else if (geom.containsAnyError()) {
			setStatus(CheckStatus.ERROR);
		} else {
			setStatus(CheckStatus.OK);
		}
	}

	@Override
	public String getText() {
		return text;
	}

	@Override
	public void visit(Renderer renderer) {
		renderer.render(geom);
	}

}
