package de.hft.stuttgart.citydoctor2.healer.genetic;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.healer.HealingPlan;

public class OrderCrossover implements Crossover<HealingPlan> {
	
	private Random r;
	
	public OrderCrossover() {
		r = new Random();
	}

	@Override
	public HealingPlan cross(HealingPlan parent1, HealingPlan parent2) {
		int size1 = parent1.getHealingMethods().size();
		int firstPoint = Math.max(0, r.nextInt(size1 - 2));
		int secondPoint = Math.min(size1, firstPoint + r.nextInt(size1 - firstPoint) + 1);
		List<HealingMethod> childMethods = new ArrayList<>(size1);
		// fill with empty values, so set(i, x) works
		for (int i = 0; i < size1; i++) {
			childMethods.add(null);
		}
		for (int i = firstPoint; i < secondPoint; i++) {
			childMethods.set(i, parent1.getHealingMethods().get(i));
		}
		int writeIndex = 0;
		for (HealingMethod m : parent2.getHealingMethods()) {
			if (!contains(childMethods, m)) {
				while (childMethods.get(writeIndex) != null) {
					writeIndex++;
				}
				childMethods.set(writeIndex, m);
			}
		}
		for (HealingMethod m : parent1.getHealingMethods()) {
			if (!contains(childMethods, m)) {
				while (childMethods.get(writeIndex) != null) {
					writeIndex++;
				}
				childMethods.set(writeIndex, m);
			}
		}
		HealingPlan child = new HealingPlan();
		child.getHealingMethods().addAll(childMethods);
		return child;
	}

	private boolean contains(List<HealingMethod> childMethods, HealingMethod m) {
		for (HealingMethod method : childMethods) {
			if (method != null && method.getClass() == m.getClass()) {
				return true;
			}
		}
		return false;
	}

}
